#include "StdAfx.h"
#include ".\tile.h"


struct tileObjectDelete
{
	template < class T >
		void operator() ( const T* ptr ) const
	{
		delete ptr;
	}
};

struct tileObjectDraw
{
	tileObjectDraw( CDC* pDC ) : _pdc( pDC ) {}
	template < class T >
		void operator() ( T* ptr ) const
	{
		ptr->DrawDC( _pdc );
	}

	CDC* _pdc;
};

struct tileObjectPick
{
	tileObjectPick( CPoint pos ) : _pos( pos ), _tile(0) {}
	template < class T >
		bool operator() ( T* ptr )
	{
		if( _tile )
			return true;

		_tile = ptr->Pick( _pos );

		return false;
	}

	CPoint _pos;
	CTile* _tile;
};

struct tileObjectPickEx
{
	tileObjectPickEx( CPoint tileidxpos ) : _pos( tileidxpos ), _tile(0) {}
	template < class T >
		bool operator() ( T* ptr )
	{
		if( _tile )
			return true;

		_tile = ptr->PickEx( _pos );

		return false;
	}

	CPoint _pos;
	CTile* _tile;
};

struct tileObjectAllClearTileIdx
{
	template < class T >
		void operator() ( T* ptr )
	{
		ptr->AllClearTileIdx();
	}
};







CTile::CTile( CPoint pos )
: _tileidx(-1), _pos( pos ), bSideTile(0)
{
	TL = T = TR = L = R = BL = B = BR = NULL;
}

CTile::~CTile()
{

}

void CTile::DrawDC( CDC* pDC )
{
	ASSERT( pDC );
//	_root->DrawDC( pDC );

	CPoint realpos = CPoint( _pos.x * xTileSize, _pos.y * yTileSize );

	CRect rect = CRect( realpos.x - xTileSize/2, realpos.y - yTileSize/2, realpos.x + xTileSize/2, realpos.y + yTileSize/2 );

	pDC->MoveTo( rect.left, rect.top );
	pDC->LineTo( rect.right, rect.top );
	pDC->LineTo( rect.right, rect.bottom );
	pDC->LineTo( rect.left, rect.bottom );
	pDC->LineTo( rect.left, rect.top );

	CString strNumber;
	strNumber.Format( "%d", _tileidx );
	pDC->TextOut( rect.left+2, rect.top+2, strNumber );

	CBrush brush;
	brush.CreateSolidBrush( RGB( 255, 0, 0 ) );
	//oldbrush = pDC->SelectObject( &brush );

	if( bSideTile )
		pDC->FillRect( &CRect( realpos.x-2, realpos.y-2, realpos.x+2, realpos.y+2 ), &brush );

	//pDC->SelectObject( oldbrush );
}

CTile* CTile::Pick( CPoint pos )
{
	CPoint realpos = CPoint( _pos.x * xTileSize, _pos.y * yTileSize );	
	CRect rect = CRect( realpos.x - xTileSize/2, realpos.y - yTileSize/2, realpos.x + xTileSize/2, realpos.y + yTileSize/2 );

	if( rect.PtInRect( pos ) )
		return this;
	return NULL;
}

CTile* CTile::PickEx( CPoint tileidxpos )
{
	if( _pos == tileidxpos )
		return this;
	return NULL;
}

void CTile::AllClearTileIdx()
{
	_tileidx = -1;
	bSideTile = false;
}



CTileGroup::CTileGroup()
{
	_components.clear();
}

CTileGroup::~CTileGroup()
{
	for_each( _components.begin(), _components.end(), tileObjectDelete() );
}

void CTileGroup::Add( CTile* tile )
{
	_components.push_back( tile );
}

void CTileGroup::DrawDC( CDC* pDC )
{
	ASSERT( pDC );
	for_each( _components.begin(), _components.end(), tileObjectDraw( pDC ) );
}

CTile* CTileGroup::Pick( CPoint pos )
{
	return for_each( _components.begin(), _components.end(), tileObjectPick( pos ) )._tile;
}

CTile* CTileGroup::PickEx( CPoint tileidxpos )
{
	return for_each( _components.begin(), _components.end(), tileObjectPickEx( tileidxpos ) )._tile;
}

void CTileGroup::AllClearTileIdx()
{
	for_each( _components.begin(), _components.end(), tileObjectAllClearTileIdx() );
}



CTileMgr::CTileMgr()
{

}

CTileMgr::~CTileMgr()
{
	delete _root;
}

void CTileMgr::DrawDC( CDC* pDC )
{
	ASSERT( pDC );

	_root->DrawDC( pDC );
}

CTile* CTileMgr::Pick( CPoint pos )
{
	return _root->Pick( pos );
}

void CTileMgr::AllClearTileIdx()
{
	_root->AllClearTileIdx();
}

void CTileMgr::PickEx( CPoint pos )
{
	CTile* ptile;

	// 콺  
	ptile = _root->Pick( pos );
	if( ptile == NULL ) 
		return;

	ptile->_tileidx = 0;

	tracePick( CPoint( ptile->_pos.x-1, ptile->_pos.y-1 ), 1 );
	tracePick( CPoint( ptile->_pos.x, ptile->_pos.y-1 ), 2 );
	tracePick( CPoint( ptile->_pos.x+1, ptile->_pos.y-1 ), 3 );
	tracePick( CPoint( ptile->_pos.x-1, ptile->_pos.y ), 4 );
	tracePick( CPoint( ptile->_pos.x+1, ptile->_pos.y ), 5 );
	tracePick( CPoint( ptile->_pos.x-1, ptile->_pos.y+1 ), 6 );
	tracePick( CPoint( ptile->_pos.x, ptile->_pos.y+1 ), 7 );
	tracePick( CPoint( ptile->_pos.x+1, ptile->_pos.y+1 ), 8 );
}

void CTileMgr::tracePick( CPoint tileidxpos, int intilenumber )
{
	CTile* ptile;
	ptile = _root->PickEx( tileidxpos );
	if( ptile == NULL ) return;
	ptile->_tileidx = intilenumber;
}

void CTileMgr::traceChkSide( CPoint tileidxpos, int& count)
{
	CTile* ptile = _root->PickEx( tileidxpos );
	if( ptile )
		if( ptile->_tileidx != -1 )
			count++;
}

void CTileMgr::ChkSide()
{
	vector< CTile* >::iterator iter;

	DWORD tick = GetTickCount();

	for( iter = _root->_components.begin(); iter != _root->_components.end(); ++iter )
	{
		CTile* p = (*iter);

		if( p->_tileidx == -1 ) continue;

		// 8 ˻縦 Ѵ.
		int count = 0;

#if 0
		traceChkSide( CPoint( p->_pos.x-1, p->_pos.y-1 ), count );
		traceChkSide( CPoint( p->_pos.x, p->_pos.y-1 ), count );
		traceChkSide( CPoint( p->_pos.x+1, p->_pos.y-1 ), count );
		traceChkSide( CPoint( p->_pos.x-1, p->_pos.y ), count );
		traceChkSide( CPoint( p->_pos.x+1, p->_pos.y ), count );
		traceChkSide( CPoint( p->_pos.x-1, p->_pos.y+1 ), count );
		traceChkSide( CPoint( p->_pos.x, p->_pos.y+1 ), count );
		traceChkSide( CPoint( p->_pos.x+1, p->_pos.y+1 ), count );
#else
		if( p->TL && p->TL->_tileidx != -1 ) count++;
		if( p->T && p->T->_tileidx != -1 ) count++;
		if( p->TR && p->TR->_tileidx != -1 ) count++;
		if( p->L && p->L->_tileidx != -1 ) count++;
		if( p->R && p->R->_tileidx != -1 ) count++;
		if( p->BL && p->BL->_tileidx != -1 ) count++;
		if( p->B && p->B->_tileidx != -1 ) count++;
		if( p->BR && p->BR->_tileidx != -1 ) count++;
#endif //

		p->bSideTile = ( count == 8 ) ? false : true;
	}

	TRACE1( "ChkSide = %d\n", GetTickCount() - tick );
}





void CTileFactory::CreateTileRoot()
{
	(CTileGroup*)CTileMgr::GetInstance()._root = new CTileGroup();
}

void CTileFactory::AddRootTile()
{
	for( int x = 1; x < 20; x++ )
	{
		for( int y = 1; y < 20; y++ )
		{
			CTileMgr::GetInstance().GetRoot()->Add( new CTile( CPoint( x, y ) ) );
		}
	}
}

void CTileFactory::CreateNeighborhood()
{
	CTile* ptile;
	CTileGroup* _root = CTileMgr::GetInstance().GetRoot();

	for( int x = 1; x < 20; x++ )
	{
		for( int y = 1; y < 20; y++ )
		{
			CTile* p = _root->PickEx( CPoint( x, y ) );

			ptile = _root->PickEx( CPoint( p->_pos.x-1, p->_pos.y-1 ) );
			if( ptile )
				p->TL = ptile;
			ptile = _root->PickEx( CPoint( p->_pos.x, p->_pos.y-1 ) );
			if( ptile )
				p->T = ptile;
			ptile = _root->PickEx( CPoint( p->_pos.x+1, p->_pos.y-1 ) );
			if( ptile )
				p->TR = ptile;
			ptile = _root->PickEx( CPoint( p->_pos.x-1, p->_pos.y ) );
			if( ptile )
				p->L = ptile;
			ptile = _root->PickEx( CPoint( p->_pos.x+1, p->_pos.y ) );
			if( ptile )
				p->R = ptile;
			ptile = _root->PickEx( CPoint( p->_pos.x-1, p->_pos.y+1 ) );
			if( ptile )
				p->BL = ptile;
			ptile = _root->PickEx( CPoint( p->_pos.x, p->_pos.y+1 ) );
			if( ptile )
				p->B = ptile;
			ptile = _root->PickEx( CPoint( p->_pos.x+1, p->_pos.y+1 ) );
			if( ptile )
				p->BR = ptile;
		}
	}
}